;################################################################################
;#										#
;# copyright (c) 2012-2013 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public				#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

sys_int:	in	r8,SREG
		push	ZH
		push	ZL
		push	XH
		push	XL
		push	r1
		push	r0

		add	precount,const_1	;pre-counter from 25,6 KHz to 100Hz
		brcc	sys_int_01
		mov	r0,count_l
		or	r0,count_h
		breq	sys_int_01
		sub	r4,const_1		;decrement timer if not zero
		sbc	r5,const_0
sys_int_01:	clr	ZL			;start value for output zero
		ldi	ZH,6			;num of channels
		ldi	XL,LOW(channel_pwm)
		ldi	XH,HIGH(channel_pwm)
sys_int_02:	lsl	ZL
		ld	r0,X+
		cp	r0,const_0
		breq	sys_int_03
		cp	r0,precount
		brcs	sys_int_03
		ori	ZL,0x01
sys_int_03:	dec	ZH
		brne	sys_int_02		;loop over all channels
		lds	r0,channel_out
		and	ZL,r0
		out	PORTC,ZL
		out	DDRC,r0			;set output mask

		;now output tone
		lds	XL,tone_num
		cpi	XL,21
		brcs	sys_int_11
		cbi	DDRB,0			;
		cbi	PORTB,0			;
		sts	tone_acc_l,const_0
		sts	tone_acc_h,const_0
		rjmp	sys_int_disp

sys_int_11:	lds	r0,tone_acc_l
		lds	r1,tone_acc_h
		ldi	ZL,LOW(tone_tab*2)
		ldi	ZH,HIGH(tone_tab*2)
		lsl	XL
		add	ZL,XL
		adc	ZH,const_0
		lpm	XL,Z+
		lpm	XH,Z+
		add	XL,r0
		adc	XH,r1
		sts	tone_acc_l,XL
		sts	tone_acc_h,XH
		sbrc	XH,7
		sbi	PORTB,0
		sbrs	XH,7
		cbi	PORTB,0
		sbi	DDRB,0

;-------------------------------------------------------------------------------
; handle display
;-------------------------------------------------------------------------------
sys_int_disp:	inc	disp_count		;display counter
		mov	XL,disp_count
		andi	XL,0x70
		cpi	XL,0x70			;prepare state
		brne	sys_int_disp_1
		clr	ZL
		sbrc	disp_count,7		;skip id we use digit 0
		ori	ZL,0x80
		eor	ZL,disp_neg		;invert if necessary
		out	PORTD,ZL		;now output data
		rjmp	sys_int_disp_e		;OK, done

sys_int_disp_1:	mov	ZL,disp_d0		;load digit 0
		sbrc	disp_count,7		;skip if we use digit 0
		mov	ZL,disp_d1		;load digit 1
		ldi	ZH,HIGH(segtable*2)	;1
		lpm	ZL,Z			;now we have the segment data
		sbrc	disp_count,7		;skip id we use digit 0
		ori	ZL,0x80
		eor	ZL,disp_neg		;invert if necessary
		out	PORTD,ZL		;now output data

sys_int_disp_e:	

;-------------------------------------------------------------------------------
; pseudo-random generator (31C)
;-------------------------------------------------------------------------------
sys_int_rand:	lds	ZL,sys_rand1		;2
		lds	ZH,sys_rand2		;2
		lds	r0,sys_rand3	;2
		lds	r1,sys_rand4	;2
		mov	XL,r1
		lsl	XL			;1
		rol	ZL
		rol	ZH
		rol	r0
		rol	r1
		brcc	sys_ird1
		ldi	XL,0x5D
		eor	ZL,XL
		ldi	XL,0xD5
		eor	ZH,XL
		ldi	XL,0x2A
		eor	r0,XL
		ldi	XL,0xD5
		eor	r1,XL

sys_ird1:	sts	sys_rand1,ZL		;2
		sts	sys_rand2,ZH		;2
		sts	sys_rand3,r0	;2
		sts	sys_rand4,r1	;2

		pop	r0
		pop	r1
		pop	XL
		pop	XH
		pop	ZL
		pop	ZH
		out	SREG,r8
		sei
		reti

tone_tab:	.db	LOW(1126),HIGH(1126)	;a	00
		.db	LOW(1193),HIGH(1193)	;b	01
		.db	LOW(1264),HIGH(1264)	;h	02
		.db	LOW(1340),HIGH(1340)	;c	03
		.db	LOW(1419),HIGH(1419)	;c'	04
		.db	LOW(1504),HIGH(1504)	;d	05
		.db	LOW(1593),HIGH(1593)	;d'	06
		.db	LOW(1688),HIGH(1688)	;e	07
		.db	LOW(1788),HIGH(1788)	;f	08
		.db	LOW(1894),HIGH(1894)	;f'	09
		.db	LOW(2007),HIGH(2007)	;g	10
		.db	LOW(2126),HIGH(2126)	;g'	11
		.db	LOW(2253),HIGH(2253)	;a	12
		.db	LOW(2387),HIGH(2387)	;b	13
		.db	LOW(2529),HIGH(2529)	;h	14
		.db	LOW(2679),HIGH(2679)	;c	15
		.db	LOW(2838),HIGH(2838)	;c'	16
		.db	LOW(3007),HIGH(3007)	;d	17
		.db	LOW(3186),HIGH(3186)	;d'	18
		.db	LOW(3375),HIGH(3375)	;e	19
		.db	LOW(3576),HIGH(3576)	;f	20
